@@ -1,5 +1,37 @@
+/*
+ * rofi
+ *
+ * MIT/X11 License
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #ifndef ROFI_MODE_PRIVATE_H
 #define ROFI_MODE_PRIVATE_H
+#include <gmodule.h>
+G_BEGIN_DECLS
+
+/** ABI version to check if loaded plugin is compatible. */
+#define ABI_VERSION    0x00000006
 
 /**
  * @param data Pointer to #Mode object.
@@ -12,13 +44,24 @@ typedef void ( *_mode_free )( Mode *data );
  * @param sw The #Mode pointer
  * @param selected_line The selected line
  * @param state The state to display [out]
+ * @param attribute_list List of extra (pango) attribute to apply when displaying. [out][null]
  * @param get_entry if it should only return the state
  *
  * Get the value for displaying.
  *
  * @return the string and state for displaying.
  */
-typedef char * ( *_mode_get_display_value )( const Mode *sw, unsigned int selected_line, int *state, int get_entry );
+typedef char * ( *_mode_get_display_value )( const Mode *sw, unsigned int selected_line, int *state, GList **attribute_list, int get_entry );
+
+/**
+ * @param sw The #Mode pointer
+ * @param selected_line The selected line
+ *
+ * Obtains the icon if available
+ *
+ * @return Get the icon
+ */
+typedef cairo_surface_t * ( *_mode_get_icon )( const Mode *sw, unsigned int selected_line, int height );
 
 /**
  * @param sw The #Mode pointer
@@ -41,14 +84,14 @@ typedef char * ( *_mode_get_completion )( const Mode *sw, unsigned int selected_
  *
  * @returns 1 when it matches, 0 if not.
  */
-typedef int ( *_mode_token_match )( const Mode *data, GRegex **tokens, unsigned int index );
+typedef int ( *_mode_token_match )( const Mode *data, rofi_int_matcher **tokens, unsigned int index );
 
 /**
  * @param sw The #Mode pointer
  *
  * Initialize the mode.
  *
- * @returns TRUE is successfull
+ * @returns TRUE is successful
  */
 typedef int ( *__mode_init )( Mode *sw );
 
@@ -91,6 +134,15 @@ typedef ModeMode ( *_mode_result )( Mode *sw, int menu_retv, char **input, unsig
  */
 typedef char* ( *_mode_preprocess_input )( Mode *sw, const char *input );
 
+/**
+ * @param sw The #Mode pointer
+ *
+ * Message to show in the message bar.
+ *
+ * @returns the (valid pango markup) message to display.
+ */
+typedef char * ( *_mode_get_message )( const Mode *sw );
+
 /**
  * Structure defining a switcher.
  * It consists of a name, callback and if enabled
@@ -98,10 +150,12 @@ typedef char* ( *_mode_preprocess_input )( Mode *sw, const char *input );
  */
 struct rofi_mode
 {
+    /** Used for external plugins. */
+    unsigned int abi_version;
     /** Name (max 31 char long) */
-    char name[32];
-    char cfg_name_key[128];
-    char *display_name;
+    char         *name;
+    char         cfg_name_key[128];
+    char         *display_name;
 
     /**
      * A switcher normally consists of the following parts:
@@ -118,11 +172,15 @@ struct rofi_mode
     _mode_token_match       _token_match;
     /** Get the string to display for the entry. */
     _mode_get_display_value _get_display_value;
+    /** Get the icon for the entry. */
+    _mode_get_icon          _get_icon;
     /** Get the 'completed' entry. */
     _mode_get_completion    _get_completion;
 
     _mode_preprocess_input  _preprocess_input;
 
+    _mode_get_message       _get_message;
+
     /** Pointer to private data. */
     void                    *private_data;
 
@@ -134,5 +192,9 @@ struct rofi_mode
     _mode_free free;
     /** Extra fields for script */
     void       *ed;
+
+    /** Module */
+    GModule    *module;
 };
+G_END_DECLS
 #endif // ROFI_MODE_PRIVATE_H
