@@ -1,6 +1,35 @@
+/*
+ * rofi
+ *
+ * MIT/X11 License
+ * Copyright © 2013-2017 Qball Cow <qball@gmpclient.org>
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ *
+ */
+
 #ifndef ROFI_MODE_H
 #define ROFI_MODE_H
-
+#include <cairo.h>
+#include "rofi-types.h"
+G_BEGIN_DECLS
 /**
  * @defgroup MODE Mode
  *
@@ -12,7 +41,7 @@
  * Type of a mode.
  * Access should be done via mode_* functions.
  */
-typedef struct rofi_mode   Mode;
+typedef struct rofi_mode Mode;
 
 /**
  * Enum used to sum the possible states of ROFI.
@@ -73,25 +102,37 @@ int mode_init ( Mode *mode );
 void mode_destroy ( Mode *mode );
 
 /**
- * @param sw The mode to query
+ * @param mode The mode to query
  *
  * Get the number of entries in the mode.
  *
  * @returns an unsigned in with the number of entries.
  */
-unsigned int mode_get_num_entries ( const Mode *sw );
+unsigned int mode_get_num_entries ( const Mode *mode );
 
 /**
  * @param mode The mode to query
  * @param selected_line The entry to query
  * @param state The state of the entry [out]
+ * @param attribute_list List of extra (pango) attribute to apply when displaying. [out][null]
  * @param get_entry If the should be returned.
  *
  * Returns the string as it should be displayed for the entry and the state of how it should be displayed.
  *
  * @returns allocated new string and state when get_entry is TRUE otherwise just the state.
  */
-char * mode_get_display_value ( const Mode *mode, unsigned int selected_line, int *state, int get_entry );
+char * mode_get_display_value ( const Mode *mode, unsigned int selected_line, int *state, GList **attribute_list, int get_entry );
+
+/**
+ * @param mode The mode to query
+ * @param selected_line The entry to query
+ * @param height The desired height of the icon.
+ *
+ * Returns the icon for the selected_line
+ *
+ * @returns allocated new cairo_surface_t if applicable
+ */
+cairo_surface_t * mode_get_icon ( const Mode *mode, unsigned int selected_line, int height );
 
 /**
  * @param mode The mode to query
@@ -124,7 +165,7 @@ ModeMode mode_result ( Mode *mode, int menu_retv, char **input, unsigned int sel
  *
  * @returns TRUE if matches
  */
-int mode_token_match ( const Mode *mode, GRegex **tokens, unsigned int selected_line );
+int mode_token_match ( const Mode *mode, rofi_int_matcher **tokens, unsigned int selected_line );
 
 /**
  * @param mode The mode to query
@@ -185,5 +226,15 @@ void mode_set_config ( Mode *mode );
  * @returns a newly allocated string
  */
 char * mode_preprocess_input ( Mode *mode, const char *input );
+
+/**
+ * @param mode The mode to query
+ *
+ * Query the mode for a user display.
+ *
+ * @return a new allocated (valid pango markup) message to display (user should free).
+ */
+char *mode_get_message ( const Mode *mode );
 /*@}*/
+G_END_DECLS
 #endif
