@@ -124,13 +124,13 @@ static int combi_mode_init ( Mode *sw )
 }
 static unsigned int combi_mode_get_num_entries ( const Mode *sw )
 {
-    const CombiModePrivateData *pd = (const CombiModePrivateData *) mode_get_private_data ( sw );
-    unsigned int length = 0;
+    const CombiModePrivateData *pd    = (const CombiModePrivateData *) mode_get_private_data ( sw );
+    unsigned int               length = 0;
     for ( unsigned int i = 0; i < pd->num_switchers; i++ ) {
         unsigned int entries = mode_get_num_entries ( pd->switchers[i].mode );
-        pd->starts[i]        = length;
-        pd->lengths[i]       = entries;
-        length+=entries;
+        pd->starts[i]  = length;
+        pd->lengths[i] = entries;
+        length        += entries;
     }
     return length;
 }
@@ -187,7 +187,7 @@ static ModeMode combi_mode_result ( Mode *sw, int mretv, char **input, unsigned
     }
     return MODE_EXIT;
 }
-static int combi_mode_match ( const Mode *sw, GRegex **tokens, unsigned int index )
+static int combi_mode_match ( const Mode *sw, rofi_int_matcher **tokens, unsigned int index )
 {
     CombiModePrivateData *pd = mode_get_private_data ( sw );
     for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
@@ -214,10 +214,13 @@ static char * combi_mgrv ( const Mode *sw, unsigned int selected_line, int *stat
     }
     for ( unsigned i = 0; i < pd->num_switchers; i++ ) {
         if ( selected_line >= pd->starts[i] && selected_line < ( pd->starts[i] + pd->lengths[i] ) ) {
-            char       * str  = mode_get_display_value ( pd->switchers[i].mode, selected_line - pd->starts[i], state, attr_list, TRUE );
+            char       * retv;
+            char       * str  = retv = mode_get_display_value ( pd->switchers[i].mode, selected_line - pd->starts[i], state, attr_list, TRUE );
             const char *dname = mode_get_display_name ( pd->switchers[i].mode );
-            char       * retv = g_strdup_printf ( "%s %s", dname, str );
-            g_free ( str );
+            if ( !config.combi_hide_mode_prefix ) {
+                retv = g_strdup_printf ( "%s %s", dname, str );
+                g_free ( str );
+            }
 
             if ( attr_list != NULL ) {
                 ThemeWidget *wid = rofi_theme_find_widget ( sw->name, NULL, TRUE );
