@@ -45,6 +45,15 @@
 
 #include <libgwater-xcb.h>
 
+#ifdef USE_NK_GIT_VERSION
+#include "nkutils-git-version.h"
+#ifdef NK_GIT_VERSION
+#define GIT_VERSION    NK_GIT_VERSION
+#endif
+#endif
+
+#include "resources.h"
+
 #include "rofi.h"
 #include "display.h"
 
@@ -62,8 +71,6 @@
 
 #include "timings.h"
 
-#include "default-theme.h"
-
 // Plugin abi version.
 // TODO: move this check to mode.c
 #include "mode-private.h"
@@ -175,7 +182,7 @@ static void run_switcher ( ModeMode mode )
         return;
     }
     curr_switcher = mode;
-    RofiViewState * state = rofi_view_create ( modi[mode], config.filter, MENU_PROMPT_COLON, process_result );
+    RofiViewState * state = rofi_view_create ( modi[mode], config.filter, 0, process_result );
     if ( state ) {
         rofi_view_set_active ( state );
     }
@@ -267,6 +274,7 @@ static void print_main_application_options ( int is_term )
     print_help_msg ( "-show", "[mode]", "Show the mode 'mode' and exit. The mode has to be enabled.", NULL, is_term );
     print_help_msg ( "-no-lazy-grab", "", "Disable lazy grab that, when fail to grab keyboard, does not block but retry later.", NULL, is_term );
     print_help_msg ( "-no-plugins", "", "Disable loading of external plugins.", NULL, is_term );
+    print_help_msg ( "-plugin-path", "", "Directory used to search for rofi plugins.", NULL, is_term );
     print_help_msg ( "-dump-config", "", "Dump the current configuration in rasi format and exit.", NULL, is_term );
     print_help_msg ( "-dump-theme", "", "Dump the current theme in rasi format and exit.", NULL, is_term );
 }
@@ -481,7 +489,7 @@ static void rofi_collect_modi_dir ( const char *base_dir )
             if ( !g_str_has_suffix ( dn, G_MODULE_SUFFIX ) ) {
                 continue;
             }
-            char    *fn  = g_build_filename ( PLUGIN_PATH, dn, NULL );
+            char    *fn  = g_build_filename ( base_dir, dn, NULL );
             GModule *mod = g_module_open ( fn, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL );
             if ( mod ) {
                 Mode *m = NULL;
@@ -529,6 +537,8 @@ static void rofi_collect_modi ( void )
     rofi_collect_modi_add ( &help_keys_mode );
 
     if ( find_arg ( "-no-plugins" ) < 0 ) {
+        find_arg_str ( "-plugin-path", &( config.plugin_path ) );
+        g_debug ( "Parse plugin path: %s", config.plugin_path );
         rofi_collect_modi_dir ( config.plugin_path );
     }
 }
@@ -551,7 +561,7 @@ static void rofi_collect_modi_destroy ( void )
             g_module_close ( mod );
         }
         if ( available_modi[i] ) {
-            mode_free ( &(available_modi[i]) );
+            mode_free ( &( available_modi[i] ) );
         }
     }
     g_free ( available_modi );
@@ -582,7 +592,7 @@ static int add_mode ( const char * token )
         Mode *sw = script_switcher_parse_setup ( token );
         if ( sw != NULL ) {
             // Add to available list, so combi can find it.
-            rofi_collect_modi_add(sw);
+            rofi_collect_modi_add ( sw );
             modi[num_modi] = sw;
             num_modi++;
         }
@@ -641,7 +651,7 @@ static gboolean startup ( G_GNUC_UNUSED gpointer data )
     TICK_N ( "Config sanity check" );
 
     if ( list_of_error_msgs != NULL ) {
-        GString *emesg = g_string_new ( "The following errors where detected when starting rofi:\n" );
+        GString *emesg = g_string_new ( "The following errors were detected when starting rofi:\n" );
         GList   *iter  = g_list_first ( list_of_error_msgs );
         int     index  = 0;
         for (; iter != NULL && index < 2; iter = g_list_next ( iter ) ) {
@@ -810,7 +820,7 @@ int main ( int argc, char *argv[] )
 
     TICK_N ( "Setup mainloop" );
 
-    bindings = nk_bindings_new ();
+    bindings = nk_bindings_new ( 0 );
     TICK_N ( "NK Bindings" );
 
     if ( !display_setup ( main_loop, bindings ) ) {
@@ -838,8 +848,9 @@ int main ( int argc, char *argv[] )
                 rofi_theme_free ( rofi_theme );
                 rofi_theme = NULL;
             }
-        }
-        else {
+        } else {
+            g_free ( config_path_new );
+            config_path_new = NULL;
             config_parse_xresource_options_file ( config_path );
         }
     }
@@ -877,18 +888,27 @@ int main ( int argc, char *argv[] )
         }
     }
     if ( rofi_theme_is_empty ( ) ) {
-        if ( rofi_theme_parse_string ( default_theme ) ) {
-            g_warning ( "Failed to parse default theme. Giving up.." );
-            if ( list_of_error_msgs ) {
-                for ( GList *iter = g_list_first ( list_of_error_msgs );
-                      iter != NULL; iter = g_list_next ( iter ) ) {
-                    g_warning ( "Error: %s%s%s",
-                                color_bold, ( (GString *) iter->data )->str, color_reset );
+        GBytes *theme_data = g_resource_lookup_data (
+            resources_get_resource (),
+            "/org/qtools/rofi/default_theme.rasi",
+            G_RESOURCE_LOOKUP_FLAGS_NONE,
+            NULL );
+        if ( theme_data ) {
+            const char *theme = g_bytes_get_data ( theme_data, NULL );
+            if ( rofi_theme_parse_string ( (const char *) theme ) ) {
+                g_warning ( "Failed to parse default theme. Giving up.." );
+                if ( list_of_error_msgs ) {
+                    for ( GList *iter = g_list_first ( list_of_error_msgs );
+                          iter != NULL; iter = g_list_next ( iter ) ) {
+                        g_warning ( "Error: %s%s%s",
+                                    color_bold, ( (GString *) iter->data )->str, color_reset );
+                    }
                 }
+                rofi_theme = NULL;
+                cleanup ();
+                return EXIT_FAILURE;
             }
-            rofi_theme = NULL;
-            cleanup ();
-            return EXIT_FAILURE;
+            g_bytes_unref ( theme_data );
         }
         rofi_theme_convert_old ();
     }
