@@ -74,24 +74,25 @@ Property *rofi_theme_property_create ( PropertyType type )
     retv->type = type;
     return retv;
 }
-Property* rofi_theme_property_copy ( Property *p)
+Property* rofi_theme_property_copy ( Property *p )
 {
     Property *retv = rofi_theme_property_create ( p->type );
     retv->name = g_strdup ( p->name );
 
-    switch ( p->type ) {
-        case P_STRING:
-          retv->value.s = g_strdup ( p->value.s );
-          break;
-        case P_LIST:
-          retv->value.list = g_list_copy_deep ( p->value.list, (GCopyFunc)g_strdup, NULL );
-          break;
-        case P_LINK:
-          retv->value.link.name = g_strdup ( p->value.link.name );
-          retv->value.link.ref  = NULL;
-          break;
-        default:
-           retv->value = p->value;
+    switch ( p->type )
+    {
+    case P_STRING:
+        retv->value.s = g_strdup ( p->value.s );
+        break;
+    case P_LIST:
+        retv->value.list = g_list_copy_deep ( p->value.list, (GCopyFunc) g_strdup, NULL );
+        break;
+    case P_LINK:
+        retv->value.link.name = g_strdup ( p->value.link.name );
+        retv->value.link.ref  = NULL;
+        break;
+    default:
+        retv->value = p->value;
     }
     return retv;
 }
@@ -114,8 +115,8 @@ void rofi_theme_property_free ( Property *p )
 void rofi_theme_reset ( void )
 {
     rofi_theme_free ( rofi_theme );
-    rofi_theme = g_slice_new0 ( ThemeWidget );
-    rofi_theme->name = g_strdup("Root");
+    rofi_theme       = g_slice_new0 ( ThemeWidget );
+    rofi_theme->name = g_strdup ( "Root" );
 }
 
 void rofi_theme_free ( ThemeWidget *widget )
@@ -138,16 +139,24 @@ void rofi_theme_free ( ThemeWidget *widget )
 /**
  * print
  */
+inline static void printf_double ( double d )
+{
+    char buf[G_ASCII_DTOSTR_BUF_SIZE];
+    g_ascii_formatd ( buf, G_ASCII_DTOSTR_BUF_SIZE, "%.4lf", d );
+    fputs ( buf, stdout );
+}
 static void rofi_theme_print_distance ( RofiDistance d )
 {
     if ( d.type == ROFI_PU_PX ) {
         printf ( "%upx ", (unsigned int) d.distance );
     }
     else if ( d.type == ROFI_PU_PERCENT ) {
-        printf ( "%f%% ", d.distance );
+        printf_double ( d.distance );
+        fputs ( "%% ", stdout );
     }
     else {
-        printf ( "%fem ", d.distance );
+        printf_double ( d.distance );
+        fputs ( "em ", stdout );
     }
     if ( d.style == ROFI_HL_DASH ) {
         printf ( "dash " );
@@ -285,11 +294,11 @@ static void rofi_theme_print_index ( ThemeWidget *widget )
             index = 4;
             for ( GList *iter = g_list_first ( list ); iter != NULL; iter = g_list_next ( iter ) ) {
                 char *name = (char *) iter->data;
-                if ( iter->prev == NULL ) {
-                    putchar ( '#' );
-                }
                 fputs ( name, stdout );
-                if ( iter->next ) {
+                if ( iter->prev == NULL && iter->next ) {
+                    putchar ( ' ' );
+                }
+                else if ( iter->next ) {
                     putchar ( '.' );
                 }
             }
@@ -320,6 +329,7 @@ static void rofi_theme_print_index ( ThemeWidget *widget )
 void rofi_theme_print ( ThemeWidget *widget )
 {
     if ( widget != NULL ) {
+        printf ( "/**\n * rofi -dump-theme output.\n * Rofi version: %s\n **/\n", PACKAGE_VERSION );
         rofi_theme_print_index ( widget );
     }
 }
@@ -369,7 +379,7 @@ void yyerror ( YYLTYPE *yylloc, const char *what, const char* s )
 static void rofi_theme_copy_property_int ( G_GNUC_UNUSED gpointer key, gpointer value, gpointer user_data )
 {
     GHashTable *table = (GHashTable *) user_data;
-    Property *p = rofi_theme_property_copy( (Property*) value);
+    Property   *p     = rofi_theme_property_copy ( (Property *) value );
     g_hash_table_replace ( table, p->name, p );
 }
 void rofi_theme_widget_add_properties ( ThemeWidget *widget, GHashTable *table )
@@ -378,7 +388,7 @@ void rofi_theme_widget_add_properties ( ThemeWidget *widget, GHashTable *table )
         return;
     }
     if ( widget->properties == NULL ) {
-        widget->properties = g_hash_table_new_full ( g_str_hash, g_str_equal, NULL, (GDestroyNotify)rofi_theme_property_free );
+        widget->properties = g_hash_table_new_full ( g_str_hash, g_str_equal, NULL, (GDestroyNotify) rofi_theme_property_free );
     }
     g_hash_table_foreach ( table, rofi_theme_copy_property_int, widget->properties );
 }
@@ -461,7 +471,7 @@ Property *rofi_theme_find_property ( ThemeWidget *widget, PropertyType type, con
     while ( widget ) {
         if ( widget->properties && g_hash_table_contains ( widget->properties, property ) ) {
             Property *p = g_hash_table_lookup ( widget->properties, property );
-            if ( p ->type == P_INHERIT ) {
+            if ( p->type == P_INHERIT ) {
                 return p;
             }
             if ( p->type == P_LINK ) {
@@ -620,6 +630,17 @@ double rofi_theme_get_double ( const widget *widget, const char *property, doubl
         }
         return p->value.f;
     }
+    // Fallback to integer if double is not found.
+    p   = rofi_theme_find_property ( wid, P_INTEGER, property, FALSE );
+    if ( p ) {
+        if ( p->type == P_INHERIT ) {
+            if ( widget->parent ) {
+                return rofi_theme_get_double ( widget->parent, property, def );
+            }
+            return def;
+        }
+        return (double)p->value.i;
+    }
     g_debug ( "Theme entry: #%s %s property %s unset.", widget->name, widget->state ? widget->state : "", property );
     return def;
 }
@@ -632,7 +653,7 @@ void rofi_theme_get_color ( const widget *widget, const char *property, cairo_t
             if ( widget->parent ) {
                 rofi_theme_get_color ( widget->parent, property, d );
             }
-            return ;
+            return;
         }
         cairo_set_source_rgba ( d,
                                 p->value.color.red,
@@ -677,7 +698,8 @@ GList *rofi_theme_get_list ( const widget *widget, const char * property, const
             if ( widget->parent ) {
                 return rofi_theme_get_list ( widget->parent, property, defaults );
             }
-        } else if ( p->type == P_LIST ) {
+        }
+        else if ( p->type == P_LIST ) {
             return g_list_copy_deep ( p->value.list, (GCopyFunc) g_strdup, NULL );
         }
     }
@@ -762,12 +784,16 @@ static char * rofi_theme_convert_color ( char *col )
 {
     char *r = g_strstrip ( col );
     if ( *r == '#' && strlen ( r ) == 9 ) {
-        char t1 = r[7];
-        char t2 = r[8];
-        r[7] = r[1];
-        r[8] = r[2];
-        r[1] = t1;
-        r[2] = t2;
+        char a1 = r[1];
+        char a2 = r[2];
+        r[1] = r[3];
+        r[2] = r[4];
+        r[3] = r[5];
+        r[4] = r[6];
+        r[5] = r[7];
+        r[6] = r[8];
+        r[7] = a1;
+        r[8] = a2;
     }
 
     return r;
@@ -802,7 +828,7 @@ void rofi_theme_convert_old ( void )
             "* { selected-normal-background: %s; }",
             "* { selected-normal-foreground: %s; }"
         };
-        for ( int i = 0; retv && retv[i]; i++ ) {
+        for ( int i = 0; retv && retv[i] && i < 5; i++ ) {
             char *str = g_strdup_printf ( conf[i], rofi_theme_convert_color ( retv[i] ) );
             rofi_theme_parse_string ( str );
             g_free ( str );
@@ -818,7 +844,7 @@ void rofi_theme_convert_old ( void )
             "* { selected-urgent-background: %s; }",
             "* { selected-urgent-foreground: %s; }"
         };
-        for ( int i = 0; retv && retv[i]; i++ ) {
+        for ( int i = 0; retv && retv[i] && i < 5; i++ ) {
             char *str = g_strdup_printf ( conf[i], rofi_theme_convert_color ( retv[i] ) );
             rofi_theme_parse_string ( str );
             g_free ( str );
@@ -834,7 +860,7 @@ void rofi_theme_convert_old ( void )
             "* { selected-active-background: %s; }",
             "* { selected-active-foreground: %s; }"
         };
-        for ( int i = 0; retv && retv[i]; i++ ) {
+        for ( int i = 0; retv && retv[i] && i < 5; i++ ) {
             char *str = g_strdup_printf ( conf[i], rofi_theme_convert_color ( retv[i] ) );
             rofi_theme_parse_string ( str );
             g_free ( str );
