@@ -81,129 +81,135 @@ typedef struct
  * Currently supports string, boolean and number (signed and unsigned).
  */
 static XrmOption xrmOptions[] = {
-    { xrm_String,  "switchers",         { .str  = &config.modi                   }, NULL,
+    { xrm_String,  "switchers",              { .str  = &config.modi                           }, NULL,
       "", CONFIG_DEFAULT },
-    { xrm_String,  "modi",              { .str  = &config.modi                   }, NULL,
+    { xrm_String,  "modi",                   { .str  = &config.modi                           }, NULL,
       "Enabled modi", CONFIG_DEFAULT },
-    { xrm_SNumber, "width",             { .snum = &config.menu_width             }, NULL,
+    { xrm_SNumber, "width",                  { .snum = &config.menu_width                     }, NULL,
       "Window width", CONFIG_DEFAULT },
-    { xrm_Number,  "lines",             { .num  = &config.menu_lines             }, NULL,
+    { xrm_Number,  "lines",                  { .num  = &config.menu_lines                     }, NULL,
       "Number of lines", CONFIG_DEFAULT },
-    { xrm_Number,  "columns",           { .num  = &config.menu_columns           }, NULL,
+    { xrm_Number,  "columns",                { .num  = &config.menu_columns                   }, NULL,
       "Number of columns", CONFIG_DEFAULT },
 
-    { xrm_String,  "font",              { .str  = &config.menu_font              }, NULL,
+    { xrm_String,  "font",                   { .str  = &config.menu_font                      }, NULL,
       "Font to use", CONFIG_DEFAULT },
-    { xrm_Number,  "borderwidth",       { .num  = &config.menu_bw                }, NULL,
+    { xrm_Number,  "borderwidth",            { .num  = &config.menu_bw                        }, NULL,
       "", CONFIG_DEFAULT },
-    { xrm_Number,  "bw",                { .num  = &config.menu_bw                }, NULL,
+    { xrm_Number,  "bw",                     { .num  = &config.menu_bw                        }, NULL,
       "Border width", CONFIG_DEFAULT },
 
-    { xrm_Number,  "location",          { .num  = &config.location               }, NULL,
+    { xrm_Number,  "location",               { .num  = &config.location                       }, NULL,
       "Location on screen", CONFIG_DEFAULT },
 
-    { xrm_Number,  "padding",           { .num  = &config.padding                }, NULL,
+    { xrm_Number,  "padding",                { .num  = &config.padding                        }, NULL,
       "Padding", CONFIG_DEFAULT },
-    { xrm_SNumber, "yoffset",           { .snum = &config.y_offset               }, NULL,
+    { xrm_SNumber, "yoffset",                { .snum = &config.y_offset                       }, NULL,
       "Y-offset relative to location", CONFIG_DEFAULT },
-    { xrm_SNumber, "xoffset",           { .snum = &config.x_offset               }, NULL,
+    { xrm_SNumber, "xoffset",                { .snum = &config.x_offset                       }, NULL,
       "X-offset relative to location", CONFIG_DEFAULT },
-    { xrm_Boolean, "fixed-num-lines",   { .num  = &config.fixed_num_lines        }, NULL,
+    { xrm_Boolean, "fixed-num-lines",        { .num  = &config.fixed_num_lines                }, NULL,
       "Always show number of lines", CONFIG_DEFAULT },
 
-    { xrm_Boolean, "show-icons",        { .snum = &config.show_icons             }, NULL,
+    { xrm_Boolean, "show-icons",             { .snum = &config.show_icons                     }, NULL,
       "Whether to load and show icons", CONFIG_DEFAULT },
 
-    { xrm_String,  "terminal",          { .str  = &config.terminal_emulator      }, NULL,
+    { xrm_String,  "terminal",               { .str  = &config.terminal_emulator              }, NULL,
       "Terminal to use", CONFIG_DEFAULT },
-    { xrm_String,  "ssh-client",        { .str  = &config.ssh_client             }, NULL,
+    { xrm_String,  "ssh-client",             { .str  = &config.ssh_client                     }, NULL,
       "Ssh client to use", CONFIG_DEFAULT },
-    { xrm_String,  "ssh-command",       { .str  = &config.ssh_command            }, NULL,
+    { xrm_String,  "ssh-command",            { .str  = &config.ssh_command                    }, NULL,
       "Ssh command to execute", CONFIG_DEFAULT },
-    { xrm_String,  "run-command",       { .str  = &config.run_command            }, NULL,
+    { xrm_String,  "run-command",            { .str  = &config.run_command                    }, NULL,
       "Run command to execute", CONFIG_DEFAULT },
-    { xrm_String,  "run-list-command",  { .str  = &config.run_list_command       }, NULL,
+    { xrm_String,  "run-list-command",       { .str  = &config.run_list_command               }, NULL,
       "Command to get extra run targets", CONFIG_DEFAULT },
-    { xrm_String,  "run-shell-command", { .str  = &config.run_shell_command      }, NULL,
+    { xrm_String,  "run-shell-command",      { .str  = &config.run_shell_command              }, NULL,
       "Run command to execute that runs in shell", CONFIG_DEFAULT },
-    { xrm_String,  "window-command",    { .str  = &config.window_command         }, NULL,
+    { xrm_String,  "window-command",         { .str  = &config.window_command                 }, NULL,
       "Command executed on accep-entry-custom for window modus", CONFIG_DEFAULT },
-    { xrm_String,  "drun-icon-theme",   { .str  = &config.drun_icon_theme        }, NULL,
+    { xrm_String,  "window-match-fields",    { .str  = &config.window_match_fields            }, NULL,
+      "Window fields to match in window mode", CONFIG_DEFAULT },
+    { xrm_String,  "drun-icon-theme",        { .str  = &config.drun_icon_theme                }, NULL,
       "Theme to use to look for icons", CONFIG_DEFAULT },
 
-    { xrm_Boolean, "disable-history",   { .num  = &config.disable_history        }, NULL,
+    { xrm_String,  "drun-match-fields",      { .str  = &config.drun_match_fields              }, NULL,
+      "Desktop entry fields to match in drun", CONFIG_DEFAULT },
+    { xrm_Boolean, "disable-history",        { .num  = &config.disable_history                }, NULL,
       "Disable history in run/ssh", CONFIG_DEFAULT },
-    { xrm_Boolean, "sort",              { .num  = &config.sort                   }, NULL,
+    { xrm_Boolean, "sort",                   { .num  = &config.sort                           }, NULL,
       "Use sorting", CONFIG_DEFAULT },
-    { xrm_Boolean, "levenshtein-sort",  { .num  = &config.levenshtein_sort       }, NULL,
+    { xrm_Boolean, "levenshtein-sort",       { .num  = &config.levenshtein_sort               }, NULL,
       "Use levenshtein sorting also for fuzzy matching", CONFIG_DEFAULT },
-    { xrm_Boolean, "case-sensitive",    { .num  = &config.case_sensitive         }, NULL,
+    { xrm_Boolean, "case-sensitive",         { .num  = &config.case_sensitive                 }, NULL,
       "Set case-sensitivity", CONFIG_DEFAULT },
-    { xrm_Boolean, "cycle",             { .num  = &config.cycle                  }, NULL,
+    { xrm_Boolean, "cycle",                  { .num  = &config.cycle                          }, NULL,
       "Cycle through the results list", CONFIG_DEFAULT },
-    { xrm_Boolean, "sidebar-mode",      { .num  = &config.sidebar_mode           }, NULL,
+    { xrm_Boolean, "sidebar-mode",           { .num  = &config.sidebar_mode                   }, NULL,
       "Enable sidebar-mode", CONFIG_DEFAULT },
-    { xrm_SNumber, "eh",                { .snum = &config.element_height         }, NULL,
+    { xrm_SNumber, "eh",                     { .snum = &config.element_height                 }, NULL,
       "Row height (in chars)", CONFIG_DEFAULT },
-    { xrm_Boolean, "auto-select",       { .num  = &config.auto_select            }, NULL,
+    { xrm_Boolean, "auto-select",            { .num  = &config.auto_select                    }, NULL,
       "Enable auto select mode", CONFIG_DEFAULT },
-    { xrm_Boolean, "parse-hosts",       { .num  = &config.parse_hosts            }, NULL,
+    { xrm_Boolean, "parse-hosts",            { .num  = &config.parse_hosts                    }, NULL,
       "Parse hosts file for ssh mode", CONFIG_DEFAULT },
-    { xrm_Boolean, "parse-known-hosts", { .num  = &config.parse_known_hosts      }, NULL,
+    { xrm_Boolean, "parse-known-hosts",      { .num  = &config.parse_known_hosts              }, NULL,
       "Parse known_hosts file for ssh mode", CONFIG_DEFAULT },
-    { xrm_String,  "combi-modi",        { .str  = &config.combi_modi             }, NULL,
+    { xrm_String,  "combi-modi",             { .str  = &config.combi_modi                     }, NULL,
       "Set the modi to combine in combi mode", CONFIG_DEFAULT },
-    { xrm_String,  "matching",          { .str  = &config.matching               }, NULL,
+    { xrm_String,  "matching",               { .str  = &config.matching                       }, NULL,
       "Set the matching algorithm. (normal, regex, glob, fuzzy)", CONFIG_DEFAULT },
-    { xrm_Boolean, "tokenize",          { .num  = &config.tokenize               }, NULL,
+    { xrm_Boolean, "tokenize",               { .num  = &config.tokenize                       }, NULL,
       "Tokenize input string", CONFIG_DEFAULT },
-    { xrm_String,  "monitor",           { .str  = &config.monitor                }, NULL,
+    { xrm_String,  "monitor",                { .str  = &config.monitor                        }, NULL,
       "", CONFIG_DEFAULT },
     /* Alias for dmenu compatibility. */
-    { xrm_String,  "m",                 { .str  = &config.monitor                }, NULL,
+    { xrm_String,  "m",                      { .str  = &config.monitor                        }, NULL,
       "Monitor id to show on", CONFIG_DEFAULT },
-    { xrm_Number,  "line-margin",       { .num  = &config.line_margin            }, NULL,
+    { xrm_Number,  "line-margin",            { .num  = &config.line_margin                    }, NULL,
       "Margin between rows *DEPRECATED*", CONFIG_DEFAULT },
-    { xrm_Number,  "line-padding",      { .num  = &config.line_padding           }, NULL,
+    { xrm_Number,  "line-padding",           { .num  = &config.line_padding                   }, NULL,
       "Padding within rows *DEPRECATED*", CONFIG_DEFAULT },
-    { xrm_String,  "filter",            { .str  = &config.filter                 }, NULL,
+    { xrm_String,  "filter",                 { .str  = &config.filter                         }, NULL,
       "Pre-set filter", CONFIG_DEFAULT },
-    { xrm_String,  "separator-style",   { .str  = &config.separator_style        }, NULL,
+    { xrm_String,  "separator-style",        { .str  = &config.separator_style                }, NULL,
       "Separator style (none, dash, solid) *DEPRECATED*", CONFIG_DEFAULT },
-    { xrm_Boolean, "hide-scrollbar",    { .num  = &config.hide_scrollbar         }, NULL,
+    { xrm_Boolean, "hide-scrollbar",         { .num  = &config.hide_scrollbar                 }, NULL,
       "Hide scroll-bar *DEPRECATED*", CONFIG_DEFAULT },
-    { xrm_Boolean, "fullscreen",        { .num  = &config.fullscreen             }, NULL,
+    { xrm_Boolean, "fullscreen",             { .num  = &config.fullscreen                     }, NULL,
       "Fullscreen", CONFIG_DEFAULT },
-    { xrm_Boolean, "fake-transparency", { .num  = &config.fake_transparency      }, NULL,
+    { xrm_Boolean, "fake-transparency",      { .num  = &config.fake_transparency              }, NULL,
       "Fake transparency *DEPRECATED*", CONFIG_DEFAULT },
-    { xrm_SNumber, "dpi",               { .snum = &config.dpi                    }, NULL,
+    { xrm_SNumber, "dpi",                    { .snum = &config.dpi                            }, NULL,
       "DPI", CONFIG_DEFAULT },
-    { xrm_Number,  "threads",           { .num  = &config.threads                }, NULL,
+    { xrm_Number,  "threads",                { .num  = &config.threads                        }, NULL,
       "Threads to use for string matching", CONFIG_DEFAULT },
-    { xrm_Number,  "scrollbar-width",   { .num  = &config.scrollbar_width        }, NULL,
+    { xrm_Number,  "scrollbar-width",        { .num  = &config.scrollbar_width                }, NULL,
       "Scrollbar width *DEPRECATED*", CONFIG_DEFAULT },
-    { xrm_Number,  "scroll-method",     { .num  = &config.scroll_method          }, NULL,
+    { xrm_Number,  "scroll-method",          { .num  = &config.scroll_method                  }, NULL,
       "Scrolling method. (0: Page, 1: Centered)", CONFIG_DEFAULT },
-    { xrm_String,  "fake-background",   { .str  = &config.fake_background        }, NULL,
+    { xrm_String,  "fake-background",        { .str  = &config.fake_background                }, NULL,
       "Background to use for fake transparency. (background or screenshot)", CONFIG_DEFAULT },
-    { xrm_String,  "window-format",     { .str  = &config.window_format          }, NULL,
+    { xrm_String,  "window-format",          { .str  = &config.window_format                  }, NULL,
       "Window Format. w (desktop name), t (title), n (name), r (role), c (class) *DEPRECATED*", CONFIG_DEFAULT },
-    { xrm_Boolean, "click-to-exit",     { .snum = &config.click_to_exit          }, NULL,
+    { xrm_Boolean, "click-to-exit",          { .snum = &config.click_to_exit                  }, NULL,
       "Click outside the window to exit", CONFIG_DEFAULT },
-    { xrm_Boolean, "show-match",        { .snum = &config.show_match             }, NULL,
+    { xrm_Boolean, "show-match",             { .snum = &config.show_match                     }, NULL,
       "Indicate how it match by underlining it.", CONFIG_DEFAULT },
-    { xrm_String,  "theme",             { .str  = &config.theme                  }, NULL,
+    { xrm_String,  "theme",                  { .str  = &config.theme                          }, NULL,
       "New style theme file", CONFIG_DEFAULT },
-    { xrm_String,  "color-normal",      { .str  = &config.color_normal           }, NULL,
+    { xrm_String,  "color-normal",           { .str  = &config.color_normal                   }, NULL,
       "Color scheme for normal row", CONFIG_DEFAULT },
-    { xrm_String,  "color-urgent",      { .str  = &config.color_urgent           }, NULL,
+    { xrm_String,  "color-urgent",           { .str  = &config.color_urgent                   }, NULL,
       "Color scheme for urgent row", CONFIG_DEFAULT },
-    { xrm_String,  "color-active",      { .str  = &config.color_active           }, NULL,
+    { xrm_String,  "color-active",           { .str  = &config.color_active                   }, NULL,
       "Color scheme for active row", CONFIG_DEFAULT },
-    { xrm_String,  "color-window",      { .str  = &config.color_window           }, NULL,
+    { xrm_String,  "color-window",           { .str  = &config.color_window                   }, NULL,
       "Color scheme window", CONFIG_DEFAULT },
-    { xrm_String,  "plugin-path",       { .str  = &config.plugin_path            }, NULL,
-      "Directory containing plugins", CONFIG_DEFAULT },
+    { xrm_Number,  "max-history-size",       { .num  = &config.max_history_size               }, NULL,
+      "Max history size (WARNING: can cause slowdowns when set to high).", CONFIG_DEFAULT },
+    { xrm_Boolean, "combi-hide-mode-prefix", { .snum = &config.combi_hide_mode_prefix         }, NULL,
+      "Hide the prefix mode prefix on the combi view.", CONFIG_DEFAULT },
 };
 
 /** Dynamic array of extra options */
@@ -220,11 +226,14 @@ void config_parser_add_option ( XrmOptionType type, const char *key, void **valu
     extra_options[num_extra_options].value.pointer = value;
     extra_options[num_extra_options].comment       = comment;
     extra_options[num_extra_options].source        = CONFIG_DEFAULT;
-    if ( type == xrm_String ) {
+    switch ( type )
+    {
+    case xrm_String:
         extra_options[num_extra_options].mem = ( (char *) ( *value ) );
-    }
-    else {
+        break;
+    default:
         extra_options[num_extra_options].mem = NULL;
+        break;
     }
 
     num_extra_options++;
@@ -232,7 +241,9 @@ void config_parser_add_option ( XrmOptionType type, const char *key, void **valu
 
 static void config_parser_set ( XrmOption *option, char *xrmValue, enum ConfigSource source )
 {
-    if ( option->type == xrm_String ) {
+    switch ( option->type )
+    {
+    case xrm_String:
         if ( ( option )->mem != NULL ) {
             g_free ( option->mem );
             option->mem = NULL;
@@ -241,14 +252,14 @@ static void config_parser_set ( XrmOption *option, char *xrmValue, enum ConfigSo
 
         // Memory
         ( option )->mem = *( option->value.str );
-    }
-    else if ( option->type == xrm_Number ) {
+        break;
+    case xrm_Number:
         *( option->value.num ) = (unsigned int) g_ascii_strtoull ( xrmValue, NULL, 10 );
-    }
-    else if ( option->type == xrm_SNumber ) {
+        break;
+    case xrm_SNumber:
         *( option->value.snum ) = (int) g_ascii_strtoll ( xrmValue, NULL, 10 );
-    }
-    else if ( option->type == xrm_Boolean ) {
+        break;
+    case xrm_Boolean:
         if ( strlen ( xrmValue ) > 0 &&
              g_ascii_strcasecmp ( xrmValue, "true" ) == 0 ) {
             *( option->value.num ) = TRUE;
@@ -256,9 +267,10 @@ static void config_parser_set ( XrmOption *option, char *xrmValue, enum ConfigSo
         else{
             *( option->value.num ) = FALSE;
         }
-    }
-    else if ( option->type == xrm_Char ) {
+        break;
+    case xrm_Char:
         *( option->value.charc ) = helper_parse_char ( xrmValue );
+        break;
     }
     option->source = source;
 }
